<?php


namespace Fastrack\Settings;


use Fastrack\Property\Property;
use GuzzleHttp\Client;
use SilverStripe\Control\Controller;
use SilverStripe\Forms\HTMLReadonlyField;
use SilverStripe\Forms\LiteralField;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\FieldType\DBDatetime;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Security\Security;
use SilverStripe\SiteConfig\SiteConfig;

class InstagramToken extends DataObject implements PermissionProvider
{

	private static $table_name = 'instagram_token';

	private static $db = [
		'UserName' => 'Varchar(255)',
		'UserId' => 'Varchar(255)',
		'AccessToken' => 'Varchar(255)',
		'AppId' => 'Varchar(255)',
		'AppSecret' => 'Varchar(255)',
		'AppCode' => 'Varchar(255)',
		'RedirectUrl' => 'Varchar(255)',
		'LongLivedAccessToken' => 'Varchar(255)',
		'LongLivedAccessTokenAge' => 'Int',
		'LongLivedAccessTokenUpdated' => 'Datetime',
	];

	private static $has_one = [
		'SiteConfig' => SiteConfig::class,
	];

	private static $has_many = [];

	private static $summary_fields = [
		'UserName',
	];

	public function getCMSFields()
	{
		$fields = parent::getCMSFields();
		$fields->addFieldToTab('Root.Main', new HTMLReadonlyField('URL', 'URL', $this->generateAuthenticationLink()));
		return $fields;
	}


	public function providePermissions()
	{
		$permisions = [
			'MANAGE_INSTAGRAM_TOKENS_ALL' => [
				'name' => 'Manage Instagram tokens',
				'category' => 'Settings'
			],
		];
		$this->extend('updatePermisions', $permisions);
		return $permisions;
	}

	public function canView($member = null)
	{
		return Permission::check(['MANAGE_INSTAGRAM_TOKENS_ALL']);
	}

	public function canEdit($member = null)
	{
		return Permission::check(['MANAGE_INSTAGRAM_TOKENS_ALL']);
	}

	public function canDelete($member = null)
	{
		return Permission::check(['MANAGE_INSTAGRAM_TOKENS_ALL']);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check(['MANAGE_INSTAGRAM_TOKENS_ALL']);
	}

	public function saveToken($code)
	{
		$token = $this->getToken($code);
		$this->AccessToken = $token->access_token;
		$this->UserId = $token->user_id;
		$this->write();
		return $token->access_token;
	}

	public function saveLongLivedToken()
	{
		$response = $this->longLivedAccessToken();
		$this->LongLivedAccessToken = $response->access_token;
		$this->LongLivedAccessTokenAge = $response->expires_in;
		$this->LongLivedAccessTokenUpdated = DBDatetime::now()->Rfc2822();
		$this->write();
	}

	public function generateAuthenticationLink()
	{
		$url = 'https://api.instagram.com/oauth/authorize?';
		$parameters = join('&', [
			'app_id=' . $this->AppId,
			'redirect_uri=' . $this->generateRedirectUrl(),
			'scope=user_profile,user_media',
			'response_type=code'
		]);
		return $url . $parameters;
	}

	private function getToken($code)
	{
		$url = 'https://api.instagram.com/oauth/access_token';
		$client = new Client();
		$response = $client->request('POST', $url, [
			'form_params' => [
				'app_id' => $this->AppId,
				'app_secret' => $this->AppSecret,
				'grant_type' => 'authorization_code',
				'redirect_uri' => $this->generateRedirectUrl(),
				'code' => $code,
			],
		]);
		return json_decode($response->getBody()->getContents());
	}

	public function generateRedirectUrl()
	{
		$siteConfig = SiteConfig::current_site_config();
		return Controller::join_links([
			'https://' . $siteConfig->PrimaryDomain,
			'instagram',
			$this->UserName,
			'redirect-url'
		]);
	}

	private function longLivedAccessToken()
	{
		$url = 'https://graph.instagram.com/access_token';
		$client = new Client();
		$response = $client->request('GET', $url, [
			'query' => [
				'grant_type' => 'ig_exchange_token',
				'client_secret' => $this->AppSecret,
				'access_token' => $this->AccessToken,
			],
		]);
		return json_decode($response->getBody()->getContents());
	}

	public function refreshAccessToken()
	{
		$url = 'https://graph.instagram.com/refresh_access_token';
		$client = new Client();
		$response = $client->request('GET', $url, [
			'query' => [
				'grant_type' => 'ig_refresh_token',
				'access_token' => $this->LongLivedAccessToken,
			],
		]);
		$jsonPayload = json_decode($response->getBody()->getContents());
		$this->LongLivedAccessToken = $jsonPayload->access_token;
		$this->LongLivedAccessTokenAge = $jsonPayload->expires_in;
		$this->LongLivedAccessTokenUpdated = DBDatetime::now()->Rfc2822();
		$this->write();
	}
}
